import { DerivedData } from "@microchip/scf-automodule-interface";
import { getModel } from "../generated_module/src/App/AppModel";
import { AppModel } from "../generated_module/src/types/AutoModuleTypes";
import { mockState } from "../generated_module/tests/AppModel.test";
import { mockAssignedModule } from "../generated_module/tests/processor.test";
import { getDerivedData } from "./DerivedData";
import { getI2CModeFunctions } from "./I2CMode";

describe("DerivedData overrideDefaultValues undefined Hardware test cases -- >", () => {
    const mockModule = mockAssignedModule(mockState());
    const model = getModel(mockModule, getDerivedData);
    if (model) {
        model.getComponentValue = (): any => {
            return "Host";
        };
        model.isComponentValue = (): any => {
            return true;
        };
    }

    const derivedData = model?.getDerivedData() as DerivedData;
    const derivedFunctions = derivedData?.getMyFunctions();
    const output = derivedFunctions?.isClientModePolling?.();

    it("Test for overrideDefaultValues result", () => {
        expect(output).toBe(false);
        expect(getI2CModeFunctions(model as AppModel)).toBeDefined();
    });
});

describe("DerivedData overrideDefaultValues undefined Hardware test cases -- >", () => {
    const mockModule = mockAssignedModule(mockState());
    const model = getModel(mockModule, getDerivedData) as AppModel;
    if (model) {
        model.getComponentValue = (): any => {
            return {
                mode: "Client",
            };
        };
    }

    const derivedData = model?.getDerivedData() as DerivedData;
    const derivedFunctions = derivedData?.getMyFunctions();
    const output = derivedFunctions?.isClientModePolling?.();

    it("Test for overrideDefaultValues result", () => {
        expect(output).toBe(false);
        //expect(getModeFunctions()).toBeDefined();
    });
});

describe("Test case for -- >", () => {
    test("success", () => {
        let mockModuleClient = mockAssignedModule(mockState());

        mockModuleClient = {
            ...mockModuleClient,
            state: {
                main: {
                    software: {
                        interruptDriven: true,
                    },
                    hardware: {
                        mode: "Client",
                    },
                },
            },
        };
        const mockModule = mockAssignedModule(mockModuleClient);
        const model = getModel(mockModule, getDerivedData);
        if (model) {
            model.getComponentValue = (): any => {
                return "Client";
            };
            model.isComponentValue = (): any => {
                return true;
            };
        }
        expect(getI2CModeFunctions(model as AppModel)).toBeDefined();
    });

    // test('error', () => {
    //     const mockModule = mockAssignedModule(mockState());
    //     const model = getModel(mockModule, getDerivedData);
    //     if (model) {
    //         model.getComponentValue = (): any => {
    //             return "error"
    //         };

    //     }
    //     expect(getI2CModeFunctions(model as AppModel)).toThrowError();
    // });
});
