import * as pin_standard from "@microchip/pin-standard";
import { getModel } from "../generated_module/src/App/AppModel";
import { AppModel, MyState } from "../generated_module/src/types/AutoModuleTypes";
import { mockAssignedModule } from "../generated_module/tests/processor.test";
import * as MyDerivedData from "./DerivedData";
import { getCompletePinData, getRowData } from "./PinsLogic";

describe("Pin Logic Test", () => {
    describe("Test for getCompleteData()", () => {
        it("Test for value", () => {
            const mockAssignedMod = mockAssignedModule({});
            const model = getModel(
                mockAssignedMod,
                MyDerivedData.getDerivedData,
            ) as AppModel;
            const expectedResult = {};

            expect(getCompletePinData(model)).toStrictEqual(expectedResult);
        });
    });

    describe("Test for getRowData() when pin standard payload is defined", () => {
        let mockAssignedMod = mockAssignedModule({});
        mockAssignedMod = {
            ...(mockAssignedMod ?? {}),
            state: {
                ...(mockAssignedMod?.state ?? {}),
                main: {
                    ...(mockAssignedMod?.state?.main ?? {}),
                },
            },
            imports: {
                ...(mockAssignedMod?.imports ?? {}),
                pin_standard: {
                    ...(mockAssignedMod?.imports?.pin_standard ?? {}),
                    payload: {
                        registers: [],
                        portmux: {
                            data: {
                                "PORTMUX.TWIROUTEA": {
                                    TWI0: "DEFAULT",
                                },
                            },
                        },
                        macroModuleId: "module2",
                    },
                },
            },
        };
        const model = getModel(mockAssignedMod, MyDerivedData.getDerivedData) as AppModel;

        it("Test for scl Pin", () => {
            const rawRowData: pin_standard.pin_row = {
                name: "scl",
                module: "TWI0",
                function: "SCL",
                direction: pin_standard.direction.bidirectional,
                filter: {
                    module: "TWI0",
                    aliasReEx: "^SCL$",
                },
                behaviour: pin_standard.behaviour.LOCK_UNLOCK,
                behaviourMeta: {
                    lockPinRegEx: "^SCL$",
                },
            };

            const processedRowData = {
                name: "scl",
                module: "TWI0",
                function: "SCL",
                direction: "bidirectional",
                filter: {
                    module: "TWI0",
                    aliasReEx: "^SCL$",
                },
                behaviour: "LOCK_UNLOCK",
                behaviourMeta: {
                    lockPinRegEx: "^SCL$",
                },
            };
            expect(getRowData(model, rawRowData)).toStrictEqual(processedRowData);
        });

        it("Test for sda Pin", () => {
            const rawRowData: pin_standard.pin_row = {
                name: "sda",
                module: "TWI0",
                function: "SDA",
                direction: pin_standard.direction.bidirectional,
                filter: {
                    module: "TWI0",
                    aliasReEx: "^SDA$",
                },
                behaviour: pin_standard.behaviour.LOCK_UNLOCK,
                behaviourMeta: {
                    lockPinRegEx: "^SDA$",
                },
            };
            const processedRowData = {
                name: "sda",
                module: "TWI0",
                function: "SDA",
                direction: "bidirectional",
                filter: {
                    module: "TWI0",
                    aliasReEx: "^SDA$",
                },
                behaviour: "LOCK_UNLOCK",
                behaviourMeta: {
                    lockPinRegEx: "^SDA$",
                },
            };
            expect(getRowData(model, rawRowData)).toStrictEqual(processedRowData);
        });
    });

    describe("Test for getRowData() when pin standard payload is undefined", () => {
        let mockAssignedMod = mockAssignedModule({});
        mockAssignedMod = {
            ...(mockAssignedMod ?? {}),
            state: {
                ...(mockAssignedMod?.state ?? {}),
                main: {
                    ...(mockAssignedMod?.state?.main ?? {}),
                },
            },
            imports: {
                ...(mockAssignedMod?.imports ?? {}),
                scf_avr8_twi_v2: {
                    ...(mockAssignedMod?.imports?.scf_avr8_twi_v2 ?? {}),
                    handle: undefined,
                    payload: undefined,
                },
                pin_standard: {
                    ...(mockAssignedMod?.imports?.pin_standard ?? {}),
                    handle: undefined,
                    payload: {
                        ...(mockAssignedMod?.imports?.pin_standard?.payload ?? {}),

                        pin_standard: undefined,
                    },
                },
            },
        };
        const model = getModel(mockAssignedMod, MyDerivedData.getDerivedData) as AppModel;

        it("Test for scl Pin", () => {
            const rawRowData: pin_standard.pin_row = {
                name: "scl",
                module: "TWI0",
                function: "SCL",
                direction: pin_standard.direction.bidirectional,
                filter: {
                    module: "TWI0",
                    aliasReEx: "^SCL$",
                },
                behaviour: pin_standard.behaviour.LOCK_UNLOCK,
                behaviourMeta: {
                    lockPinRegEx: "^SCL$",
                },
            };

            const processedRowData = {
                name: "scl",
                module: "TWIundefined",
                function: "SCL",
                direction: "bidirectional",
                filter: {
                    module: "TWIundefined",
                    aliasReEx: "^SCL$",
                },
                behaviour: "LOCK_UNLOCK",
                behaviourMeta: {
                    lockPinRegEx: "^SCL$",
                },
            };
            expect(getRowData(model, rawRowData)).toStrictEqual(processedRowData);
        });

        it("Test for sda Pin", () => {
            const rawRowData: pin_standard.pin_row = {
                name: "sda",
                module: "TWI0",
                function: "SDA",
                direction: pin_standard.direction.bidirectional,
                filter: {
                    module: "TWI0",
                    aliasReEx: "^SDA$",
                },
                behaviour: pin_standard.behaviour.LOCK_UNLOCK,
                behaviourMeta: {
                    lockPinRegEx: "^SDA$",
                },
            };
            const processedRowData = {
                name: "sda",
                module: "TWIundefined",
                function: "SDA",
                direction: "bidirectional",
                filter: {
                    module: "TWIundefined",
                    aliasReEx: "^SDA$",
                },
                behaviour: "LOCK_UNLOCK",
                behaviourMeta: {
                    lockPinRegEx: "^SDA$",
                },
            };
            expect(getRowData(model, rawRowData)).toStrictEqual(processedRowData);
        });
    });

    describe("Test for getRowData() when hardware is undefined", () => {
        let mockAssignedMod = mockAssignedModule({});
        mockAssignedMod = {
            ...(mockAssignedMod ?? {}),
            state: {},
            imports: {
                ...(mockAssignedMod?.imports ?? {}),
                scf_avr8_twi_v2: {
                    ...(mockAssignedMod?.imports?.scf_avr8_twi_v2 ?? {}),
                    payload: undefined,
                },
                pin_standard: {
                    ...(mockAssignedMod?.imports?.pin_standard ?? {}),
                    payload: {
                        registers: [],
                        portmux: {
                            data: {
                                "PORTMUX.TWIROUTEA": {
                                    TWI0: "DEFAULT",
                                    TWI1: "DEFAULT",
                                },
                            },
                        },
                        macroModuleId: "module2",
                    },
                },
            },
        };
        const model = getModel(mockAssignedMod, MyDerivedData.getDerivedData) as AppModel;

        it("Test for scl Pin", () => {
            const rawRowData: pin_standard.pin_row = {
                name: "scl",
                module: "TWI0",
                function: "SCL",
                direction: pin_standard.direction.bidirectional,
                filter: {
                    module: "TWI0",
                    aliasReEx: "^SCL$",
                },
                behaviour: pin_standard.behaviour.LOCK_UNLOCK,
                behaviourMeta: {
                    lockPinRegEx: "^SCL$",
                },
            };

            const processedRowData = {
                name: "scl",
                module: "TWIundefined",
                function: "SCL",
                direction: "bidirectional",
                filter: {
                    module: "TWIundefined",
                    aliasReEx: "^SCL$",
                },
                behaviour: "LOCK_UNLOCK",
                behaviourMeta: {
                    lockPinRegEx: "^SCL$",
                },
            };
            expect(getRowData(model, rawRowData)).toStrictEqual(processedRowData);
        });

        it("Test for sda Pin", () => {
            const rawRowData: pin_standard.pin_row = {
                name: "sda",
                module: "TWI0",
                function: "SDA",
                direction: pin_standard.direction.bidirectional,
                filter: {
                    module: "TWI0",
                    aliasReEx: "^SDA$",
                },
                behaviour: pin_standard.behaviour.LOCK_UNLOCK,
                behaviourMeta: {
                    lockPinRegEx: "^SDA$",
                },
            };
            const processedRowData = {
                name: "sda",
                module: "TWIundefined",
                function: "SDA",
                direction: "bidirectional",
                filter: {
                    module: "TWIundefined",
                    aliasReEx: "^SDA$",
                },
                behaviour: "LOCK_UNLOCK",
                behaviourMeta: {
                    lockPinRegEx: "^SDA$",
                },
            };
            expect(getRowData(model, rawRowData)).toStrictEqual(processedRowData);
        });
    });

    describe("Test for getRowData() when Device meta is Defined", () => {
        let mockAssignedMod = mockAssignedModule({});
        mockAssignedMod = {
            ...(mockAssignedMod ?? {}),
            state: {
                ...(mockAssignedMod?.state ?? {}),
                main: {
                    ...(mockAssignedMod?.state?.main ?? {}),
                    hardware: {},
                },
            },
        };
        const model = getModel(mockAssignedMod, MyDerivedData.getDerivedData) as AppModel;
        if (model?.getImports()?.device_meta.payload) {
            model.getImports().device_meta = {
                ...model.getImports().device_meta,
                payload: {
                    deviceName: "ATtiny817",
                },
            };
        }
        // model.getImportValue = (): any => {
        //     return "ATtiny817";
        // };

        it("Test for SCL Pin", () => {
            const rawRowData: pin_standard.pin_row = {
                name: "scl",
                module: "TWI0",
                function: "SCL",
                direction: pin_standard.direction.bidirectional,
                filter: {
                    module: "TWI0",
                    aliasReEx: "^SCL$",
                },
                behaviour: pin_standard.behaviour.LOCK_UNLOCK,
                behaviourMeta: {
                    lockPinRegEx: "^SCL$",
                },
            };

            expect(getRowData(model, rawRowData)).toBeDefined();
        });
    });
    describe("Test for getRowData() when Device meta is undefined", () => {
        let mockAssignedMod = mockAssignedModule({});
        mockAssignedMod = {
            ...(mockAssignedMod ?? {}),
            state: {
                ...(mockAssignedMod?.state ?? {}),
                main: {
                    ...(mockAssignedMod?.state?.main ?? {}),
                    hardware: {},
                },
            },
        };
        const model = getModel(mockAssignedMod, MyDerivedData.getDerivedData) as AppModel;
        model.getImportValue = (): any => {
            return undefined;
        };

        it("Test for SCL Pin", () => {
            const rawRowData: pin_standard.pin_row = {
                name: "scl",
                module: "TWI0",
                function: "SCL",
                direction: pin_standard.direction.bidirectional,
                filter: {
                    module: "TWI0",
                    aliasReEx: "^SCL$",
                },
                behaviour: pin_standard.behaviour.LOCK_UNLOCK,
                behaviourMeta: {
                    lockPinRegEx: "^SCL$",
                },
            };

            expect(getRowData(model, rawRowData)).toBeDefined();
        });
    });

    describe("Test for getRowData() when Device meta is non AVR device", () => {
        let mockAssignedMod = mockAssignedModule({});
        mockAssignedMod = {
            ...(mockAssignedMod ?? {}),
            state: {
                ...(mockAssignedMod?.state ?? {}),
                main: {
                    ...(mockAssignedMod?.state?.main ?? {}),
                    hardware: {},
                },
            },
        };
        const model = getModel(mockAssignedMod, MyDerivedData.getDerivedData) as AppModel;
        model.getImportValue = (): any => {
            return "ATtiny817";
        };

        it("Test for SCL Pin", () => {
            const rawRowData: pin_standard.pin_row = {
                name: "scl",
                module: "TWI0",
                function: "SCL",
                direction: pin_standard.direction.bidirectional,
                filter: {
                    module: "TWI0",
                    aliasReEx: "^SCL$",
                },
                behaviour: pin_standard.behaviour.LOCK_UNLOCK,
                behaviourMeta: {
                    lockPinRegEx: "^SCL$",
                },
            };

            expect(getRowData(model, rawRowData)).toBeDefined();
        });
    });

    describe("Test for getCompleteData()", () => {
        const mockState: MyState = {
            main: {
                software: {},
            },
        };
        it("Test for value", () => {
            const mockAssignedMod = mockAssignedModule(mockState);
            const model = getModel(
                mockAssignedMod,
                MyDerivedData.getDerivedData,
            ) as AppModel;
            expect(getCompletePinData(model)).toBeDefined();
        });
        test("Device meta is defined", () => {
            const mockModule = mockAssignedModule(mockState);
            const model = getModel(mockModule, MyDerivedData.getDerivedData) as AppModel;

            if (model) {
                model.getImportValue = (): any => {
                    return "AVR64EA48";
                };
            }
            expect(getCompletePinData(model)).toBeDefined();
        });
        test("Device meta is undefined", () => {
            const mockModule = mockAssignedModule(mockState);
            const model = getModel(mockModule, MyDerivedData.getDerivedData) as AppModel;

            if (model) {
                model.getImportValue = (): any => {
                    return undefined;
                };
            }
            expect(getCompletePinData(model)).toBeDefined();
        });
    });
});
