import { behaviour, pin_row } from "@microchip/pin-standard";
import { Meta } from "@microchip/scf-device/lib/Meta";
import { AppModel } from "../generated_module/src/types/AutoModuleTypes";
// import { PCPHelper } from "../generated_module/src/pins/PCPHelper";
// import { pingroups_data, pins_group } from "@microchip/pin-standard/lib/pin-standard";
// import { getKeys } from "../generated_module/src/Utils";
// import { Dictionary } from "@microchip/scf-common/lib/Processor";

// let retRowJSONdata: pin_row = rowData;
// const mod = appModel.getHardware()?.getName();
// const pinPayload = appModel?.getImportValue("pin_standard") ?? {};

// provide complete dynamic data for all pin rows. It overrides static pindata.json
export const getCompletePinData = (appModel: AppModel): any => {
    // way to programatically override the json data
    return {};
};

// overrides pin data for a particular row based on model.
export function getRowData(appModel: AppModel, rowData: pin_row): pin_row {
    const TwiInstanceName = appModel.getHardware()?.getPeripheral()?.instance;
    const hardwareName = appModel.getHardware()?.getName() ?? "TWI0";
    const pinPayload = appModel.getImportValue("pin_standard") ?? {};
    const deviceMeta: Meta = appModel.getImportValue("device_meta");
    // To fetch deviceName
    const deviceName = deviceMeta?.deviceName ?? "ATtiny..*1d";
    // const targetportmuxreg = "PORTMUX.USARTROUTEA";
    // const targetportmuxsetting = "USART0";
    //const portmuxgrp: string = filterGroupName(pinPayload,targetportmuxreg,targetportmuxsetting,);
    if (!/^ATtiny..*1\d$/.test(deviceName)) {
        if (rowData.name === "scl") {
            rowData = {
                ...rowData,
                module: "TWI" + TwiInstanceName,
                filter: {
                    ...rowData.filter,
                    module: "TWI" + TwiInstanceName,
                },
            };
        }
        if (rowData.name === "sda") {
            rowData = {
                ...rowData,
                module: "TWI" + TwiInstanceName,
                filter: {
                    ...rowData.filter,
                    module: "TWI" + TwiInstanceName,
                },
            };
        }
    } else {
        const targetPortmuxReg = "PORTMUX.CTRLB";
        const targetPortmuxSetting = hardwareName;
        const portmuxGrp =
            pinPayload.portmux?.data?.[targetPortmuxReg]?.[hardwareName] ?? "";

        if (rowData.name === "scl") {
            const groupvsplockregex = {};
            groupvsplockregex["DEFAULT"] = `SCL_DEF`;
            groupvsplockregex["ALTERNATE"] = `SCL_ALT`;
            const lockPinRegEx = groupvsplockregex[portmuxGrp];
            rowData = {
                ...rowData,
                module: hardwareName,
                filter: {
                    ...rowData.filter,
                    module: hardwareName,
                    aliasReEx: "^SCL$",
                },
                behaviourMeta: {
                    lockPinRegEx: lockPinRegEx,
                },
                behaviour: behaviour.PORTMUX,
                groupsdata: {
                    groups: [
                        {
                            name: "DEFAULT",
                            datapath: `${targetPortmuxReg}%${targetPortmuxSetting}`,
                            filter: {
                                aliasReEx: "SCL_DEF",
                            },
                        },
                        {
                            name: "ALTERNATE",
                            datapath: `${targetPortmuxReg}%${targetPortmuxSetting}`,
                            filter: {
                                aliasReEx: "SCL_ALT",
                            },
                        },
                    ],
                },
            };
        } else if (rowData.name === "sda") {
            const groupvsplockregex = {};
            groupvsplockregex["DEFAULT"] = `SDA_DEF`;
            groupvsplockregex["ALTERNATE"] = `SDA_ALT`;
            const lockPinRegEx = groupvsplockregex[portmuxGrp];
            rowData = {
                ...rowData,
                module: hardwareName,
                filter: {
                    ...rowData.filter,
                    module: hardwareName,
                    aliasReEx: "^SDA$",
                },
                behaviourMeta: {
                    lockPinRegEx: lockPinRegEx,
                },
                behaviour: behaviour.PORTMUX,
                groupsdata: {
                    groups: [
                        {
                            name: "DEFAULT",
                            datapath: `${targetPortmuxReg}%${targetPortmuxSetting}`,
                            filter: {
                                aliasReEx: "SDA_DEF",
                            },
                        },
                        {
                            name: "ALTERNATE",
                            datapath: `${targetPortmuxReg}%${targetPortmuxSetting}`,
                            filter: {
                                aliasReEx: "SDA_ALT",
                            },
                        },
                    ],
                },
            };
        }
    }

    return rowData;
}
